﻿#nullable enable

using FinkiChattery.Contracts.Questioning.GetQuestionState;
using FinkiChattery.Persistence.Repositories.Contracts;
using System.Collections.Generic;
using System.Linq;

namespace FinkiChattery.Api.ApplicationServices.Questioning
{
    public static class QuestionMapper
    {
        public static QuestionStateResponse ToQuestionStateResponse(this QuestionStateDto questionState)
        {
            IEnumerable<AnswerQuestionStateResponse> answers = Enumerable.Empty<AnswerQuestionStateResponse>();

            if (questionState.AnswersDto.Any())
            {
                answers = questionState.AnswersDto.Select(x =>
                {
                    IEnumerable<AnswerResponseQuestionStateResponse> answerResponses = Enumerable.Empty<AnswerResponseQuestionStateResponse>();

                    if (x.AnswerResponsesDto.Any())
                    {
                        answerResponses = x.AnswerResponsesDto.Select(y =>
                        {
                            var answerResponseStudent = new AnswerResponseStudentQuestionStateResponse(y.StudentDto.Id, y.StudentDto.Uid, y.StudentDto.Index, y.StudentDto.ImageUrl);

                            return new AnswerResponseQuestionStateResponse(y.Id, y.Uid, y.Text, y.CreatedOn, answerResponseStudent);
                        });
                    }

                    var answerStudent = new AnswerStudentQuestionStateResponse(x.StudentDto.Id, x.StudentDto.Uid, x.StudentDto.Index, x.StudentDto.ImageUrl);

                    return new AnswerQuestionStateResponse(x.Id, x.Uid, x.Text, x.CorrectAnswer, x.CreatedOn, x.UpvotesCount, answerStudent, answerResponses);
                });
            }

            IEnumerable<QuestionCategoryQuestionStateResponse> questionCategories = Enumerable.Empty<QuestionCategoryQuestionStateResponse>();

            if (questionState.CategoriesDto.Any())
            {
                questionCategories = questionState.CategoriesDto.Select(x => new QuestionCategoryQuestionStateResponse(x.Id, x.Uid, x.Name));
            }

            TeamQuestionStateResponse? team = null;

            if (questionState.TeamDto != null)
            {
                team = new TeamQuestionStateResponse(questionState.TeamDto.Id, questionState.TeamDto.Uid, questionState.TeamDto.Name);
            }

            var student = new StudentQuestionStateResponse(questionState.StudentDto.Id, questionState.StudentDto.Uid, questionState.StudentDto.Index, questionState.StudentDto.ImageUrl);

            return new QuestionStateResponse(questionState.Id, questionState.Uid, questionState.Title, questionState.Text, questionState.CreatedOn, questionState.Views, questionState.LastActiveOn, student, answers, questionCategories, team);
        }
    }
}
